import java.io.IOException;
import java.io.File;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathFactory;
import javax.xml.xpath.XPath;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.Result;
import javax.xml.transform.Transformer;

import org.w3c.dom.Attr;
import org.w3c.dom.CDATASection;
import org.w3c.dom.Comment;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentType;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.ProcessingInstruction;
import org.w3c.dom.Text;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;


public class MergeXmlDemo {

    public static void main(String[] args) throws Exception {
	// proper error/exception handling omitted for brevity
	File file1 = new File("merge1.xml");
	File file2 = new File("merge2.xml");
	Document doc = merge("/frida", file1, file2);
	print(doc);
    }

    private static Document merge(String expression,
				  File... files) throws Exception {
	XPathFactory xPathFactory = XPathFactory.newInstance();
	XPath xpath = xPathFactory.newXPath();
	XPathExpression compiledExpression = xpath
	    .compile(expression);
	return merge(compiledExpression, files);
    }

    private static Document merge(XPathExpression expression,
				  File... files) throws Exception {
	DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory
	    .newInstance();
	docBuilderFactory
	    .setIgnoringElementContentWhitespace(true);
	DocumentBuilder docBuilder = docBuilderFactory
	    .newDocumentBuilder();
	Document base = docBuilder.parse(files[0]);

	Node results = (Node) expression.evaluate(base,
						  XPathConstants.NODE);
	if (results == null) {
	    throw new IOException(files[0]
				  + ": expression does not evaluate to node");
	}

	XPathFactory xpf = XPathFactory.newInstance();
	XPath xp = xpf.newXPath();
	XPathExpression xpe = xp.compile("/frida/datoGenerert");
	results.removeChild((Node) xpe.evaluate(base, XPathConstants.NODE));
	xpe = xp.compile("/frida/skjemaversjon");
	results.removeChild((Node) xpe.evaluate(base, XPathConstants.NODE));

	for (int i = 1; i < files.length; i++) {
	    Document merge = docBuilder.parse(files[i]);
	    xpe = xp.compile("/frida/forskningsresultat");
	    Node tree = (Node) xpe.evaluate(merge,
					    XPathConstants.NODE);
	    tree = base.importNode(tree, true);
	    results.appendChild(tree);
	    /*
	    while (nextResults.hasChildNodes()) {
		Node kid = nextResults.getFirstChild();
		nextResults.removeChild(kid);
		kid = base.importNode(kid, true);
		results.appendChild(kid);
	    }
	    */
	}

	NamedNodeMap nm = results.getAttributes();
	nm.removeNamedItem("xsi:noNamespaceSchemaLocation");
	nm.removeNamedItem("xmlns:xsi");

	base.setXmlStandalone(true);

	return base;
    }

    private static void print(Document doc) throws Exception {
	TransformerFactory transformerFactory = TransformerFactory
	    .newInstance();
    Transformer transformer = transformerFactory
        .newTransformer();
    DOMSource source = new DOMSource(doc);
    Result result = new StreamResult(System.out);
    transformer.transform(source, result);
  }

}
