package no.usit.fridaws;

/**
 * Sorteringsfunksjoner.
 *
 */
public class Filter {

    /**
     * Returnerer en bit SQL som kan brukes sammen med "order by", gitt
     * navnet paa et sorteringskriterium.
     *
     * Forutsetter at SQL-koden den limes inn i definerer v som
     * vitenskapeligarbeid tabellen.
     *
     * Returnerer null hvis angitt sorteringskriterium ikke finnes.
     *
     * @param sortering navnet paa et sorteringskriterium
     * @return SQL order-by spesifikasjon
     */
    static String lagVarbeidSortering(String sortering) {
		sortering = sortering.toUpperCase();
	
		String mapping[][] = { 
		    { "PERSON_TITTEL",
		      "v.sortering_forfatter asc, v.titteltekst_orginal asc" },
		    { "AAR_PERSON_TITTEL",
		      "v.arstall desc, v.sortering_forfatter asc, "
		      + "v.titteltekst_orginal asc" },
		    { "HOVEDKAT_PERSON_TITTEL",
		      "v.varbeidhovedkatkode asc, v.sortering_forfatter asc,"
		      + " v.titteltekst_orginal asc" },
		    { "UNDERKAT_PERSON_TITTEL",
		      "v.varbeidunderkatkode asc, v.sortering_forfatter asc,"
		      + " v.titteltekst_orginal asc" }
		};
	
		for (int i = 0; i < mapping.length; i++) {
		    if (mapping[i][0].equals(sortering))
			return mapping[i][1];
		}
	
		return null;
    }

    /**
     * Returnerer en bit SQL som kan brukes sammen med "order by", gitt
     * navnet paa et sorteringskriterium.
     * 
     * Forutsetter at SQL-koden den limes inn i definerer pe som
     * presentasjon_enhet tabellen.
     *
     * Returnerer null hvis angitt sorteringskriterium ikke finnes.
     *
     * @param sortering navnet paa et sorteringskriterium
     * @return SQL order-by spesifikasjon
     */
    static String lagPersonSortering(String sortering) {
		sortering = sortering.toUpperCase();
	
		String mapping[][] = { 
		    { "STED",
		      "pe.institusjonsnr asc, pe.avdnr asc, " +
		      "pe.undavdnr asc, pe.gruppenr asc" }
		};
	
		for (int i = 0; i < mapping.length; i++) {
		    if (mapping[i][0].equals(sortering))
			return mapping[i][1];
		}
	
		return null;
	    }
	
	    /**
	     * Returnerer alltid null (ikke implementert)
	     *
	     */
	    static String lagPresentasjonSortering(String sortering) {
		return null;
    }

}
