package no.usit.fridaws;

import java.sql.Connection;
import java.sql.Statement;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import javax.sql.DataSource;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.QueryParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Variant;

import java.util.regex.Pattern;
import java.util.regex.Matcher;

import org.jboss.resteasy.annotations.cache.Cache;

import net.sf.json.xml.*;

/**
 * Person RPC-tjeneste.
 */
@Path("/hentPerson")
public class Person
{

    /**
     * Returnerer en person basert paa personens loepenummer, eller
     * brukernavn og eierkode.
     *
     * @param navn personens brukernavn
     * @param eierkode stedet brukernavnet hører hjemme
     * @param lopenr personens unike loepenummer i databasen
     * @param format "xml" eller "json" for å angi output format
     * @return en person ressurs
     */
    @GET
    @Cache(maxAge=0)
    public Response hentPerson(
		@QueryParam("eierkode") String eierkode,
		@QueryParam("navn") String navn,
		@QueryParam("lopenr") Long lopenr,
		@QueryParam("format") String format
	) {
		if (format == null)
		    format = "xml";
		else
		    format = format.toLowerCase();
		if (eierkode != null)
		    eierkode = eierkode.toUpperCase();
	
		if (!InputValidering.okPerson(navn, eierkode, lopenr) ||
		    !(format.equals("xml") ^ format.equals("json"))) {
		    return Response.status(400).build();
		}
	
		String output = null;
		Connection conn = null;
		PreparedStatement stmt = null;
	
		try {
		    conn = Database.getConnection();
		    String sql;
		    
		    if (navn != null && eierkode != null) {
		    	lopenr = Database.getPersonLopeNr(navn, eierkode);
		    }
		    if (lopenr == null) {
		    	return Response.status(404).build();
		    }
		    
		    sql =
				"SELECT tekst_xml " +
				"FROM   presentasjon_xml x, person p " +
				"WHERE  x.presentasjonslopenr = p.presentasjonslopenr " + 
				"  AND  p.personlopenr = :1 " +
				"  AND  tekst_xml IS NOT null";
		    stmt = conn.prepareStatement(sql);
		    stmt.setLong(1, lopenr);
	
		    ResultSet rs = stmt.executeQuery();
		    if (rs.next()) {
		    	output = rs.getString(1);
		    } else {
		    	return Response.status(404).build();
		    }
		    
		} catch (SQLException e) {
		    Database.loggFeil(e);
		    return Response.status(500).build();
	
		} finally {
		    try {
			if (conn != null)
			    conn.close();
		    } catch (SQLException e) {
		    	Database.loggFeil(e);
		    	return Response.status(500).build();
		    }
		}
	
		MediaType outputFormat = MediaType.APPLICATION_XML_TYPE;
	
		if (format.equals("json")) {
		    XMLSerializer xs = new XMLSerializer();
		    output = xs.read(output).toString();
		    outputFormat = MediaType.APPLICATION_JSON_TYPE;
		}
		    
		return Response
		    .status(200)
		    .entity(output)
		    .variant(new Variant(outputFormat,
					 new java.util.Locale("no_NO"),
					 null))
		    .build();
    }
}
