package no.usit.fridaws;

import java.sql.Connection;
import java.sql.Statement;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import javax.sql.DataSource;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.QueryParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Variant;

import java.util.regex.Pattern;
import java.util.regex.Matcher;

import org.jboss.resteasy.annotations.cache.Cache;

import net.sf.json.xml.*;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * Presentasjon RPC-tjeneste.
 */
@Path("/hentPresentasjon")
public class Presentasjon
{
    private static final Logger logger =
	LoggerFactory.getLogger(Presentasjon.class);


    /**
     * Returnerer en presentasjon, gitt et loepenr.  Hvis type er
     * oppgitt, og presentasjonen ikke er av denne typen, returneres
     * en 404 respons.
     *
     * @param lopenr presentasjonens unike loepenummer i databasen
     * @param type presentasjonstype
     * @param format "xml" eller "json" for å angi output format
     * @return en presentasjon
     */
    @GET
    @Cache(maxAge=0)
    public Response hentPerson(
		@QueryParam("lopenr") Long lopenr,
		@QueryParam("type") String type,
		@QueryParam("format") String format
	) {
		if (format == null) {
		    format = "xml";
		} else {
		    format = format.toLowerCase();
		}
		if (type != null) {
		    type = type.toUpperCase();
		}
	
		if (lopenr == null ||
		    !(format.equals("xml") ^ format.equals("json"))) {
		    return Response.status(400).build();
		}
	
		String output = null;
		Connection conn = null;
		PreparedStatement stmt = null;
	
		try {
		    conn = Database.getConnection();
		    String sql;
		    
		    sql =
				"SELECT tekst_xml " +
				"FROM   presentasjon_xml x, " +
				"       presentasjon p " +
				"WHERE  p.presentasjonslopenr = x.presentasjonslopenr " +
				"  AND  p.presentasjonslopenr = ? ";
		    
		    if (type != null) {
		    	sql += " AND presentasjonstypekode = ? ";
		    }
		    stmt = conn.prepareStatement(sql);
		    stmt.setLong(1, lopenr);
		    if (type != null) {
		    	stmt.setString(2, type);
		    }
	
		    logger.debug(sql);
	
		    ResultSet rs = stmt.executeQuery();
		    if (rs.next()) {
		    	output = rs.getString(1);
		    } else {
		    	return Response.status(404).build();
		    }
		    
		} catch (SQLException e) {
		    Database.loggFeil(e);
		    return Response.status(500).build();
	
		} finally {
		    try {
		    	if (conn != null) {
		    		conn.close();
		    	}
		    } catch (SQLException e) {
		    	Database.loggFeil(e);
		    	return Response.status(500).build();
		    }
		}
	
		MediaType outputFormat = MediaType.APPLICATION_XML_TYPE;
	
		if (format.equals("json")) {
		    XMLSerializer xs = new XMLSerializer();
		    output = xs.read(output).toString();
		    outputFormat = MediaType.APPLICATION_JSON_TYPE;
		}
		    
		return Response
		    .status(200)
		    .entity(output)
		    .variant(new Variant(outputFormat,
					 new java.util.Locale("no_NO"),
					 null))
		    .build();
    }

}
