package no.usit.fridaws;

import java.sql.Connection;
import java.sql.Statement;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Blob;

import javax.sql.DataSource;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.QueryParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Variant;

import java.util.regex.Pattern;
import java.util.regex.Matcher;

import org.jboss.resteasy.annotations.cache.Cache;

import net.sf.json.xml.*;

/**
 * Presentasjonsbilde RPC-tjeneste.
 */
@Path("/hentPresentasjonBilde")
public class PresentasjonBilde
{

    /**
     * Returnerer logoen til en presentasjon
     *
     * @param lopenr presentasjonens lopenr i basen
     * @return bilde til en presentasjon
     */
    @GET
    @Cache(maxAge=0)
    @Produces("image/jpeg")
    public Response hentPresentasjonBilde(
    		@QueryParam("lopenr") Long lopenr
	) {
		if (lopenr == null) {
		    return Response.status(400).build();
		}
			    
		Blob output = null;
		Connection conn = null;
		PreparedStatement stmt = null;
	
		try {
		    conn = Database.getConnection();
		    String sql;
		    
		    sql =
		    	"SELECT logo " +
		    	"FROM   presentasjon " +
		    	"WHERE  presentasjonslopenr = :1";
		    stmt = conn.prepareStatement(sql);
		    stmt.setLong(1, lopenr);
	
		    ResultSet rs = stmt.executeQuery();
		    if (rs.next()) {
		    	output = rs.getBlob(1);
				if (output == null) {
				    return Response.status(404).build();
				}
		    } else {
		    	return Response.status(404).build();
		    }
		    
		} catch (SQLException e) {
		    Database.loggFeil(e);
		    return Response.status(500).build();
		    
		} finally {
		    try {
		    	if (conn != null) {
		    		conn.close();
		    	}
		    } catch (SQLException e) {
		    	Database.loggFeil(e);
		    	return Response.status(500).build();
		    }
		}
	
		try {
		    return Response
			.status(200)
			.entity(output.getBinaryStream())
			.build();
		} catch (SQLException e) {
		    Database.loggFeil(e);
		    return Response.status(500).build();
		}
    }
}
