package no.usit.fridaws;

import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.List;
import java.util.Date;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathFactory;
import javax.xml.xpath.XPath;

import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.Result;
import javax.xml.transform.Transformer;

import org.w3c.dom.Attr;
import org.w3c.dom.CDATASection;
import org.w3c.dom.Comment;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentType;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.ProcessingInstruction;
import org.w3c.dom.Text;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import java.text.SimpleDateFormat;

/**
 * Klasse for håndtering av XML.
 *
 */
public class Xml {

    /**
     * Exception som kan oppstå i forbindelse med håndtering av XML.
     *
     */
    public static class XmlException extends Exception {
	XmlException(String msg, Throwable ex) { super(msg, ex); }
    }

    /**
     * Lager XPathExpression fra en xpath spesifisert gjennom en streng.
     *
     * @param path streng som representerer en XPath
     * @return et XPathExpression objekt
     */
    public static XPathExpression stringToXPath(String path)
	throws XPathExpressionException {
		return XPathFactory
		    .newInstance()
		    .newXPath()
		    .compile(path);
    }

    /**
     * Syr sammen en liste av XML-biter til et dokument.
     *
     * @param skjemaURI URI til skjema for resultat-XML
     * @param partsPath biter med XML
     * @return sammensatt XML
     */
    public static String join(
    	String skjemaURI, XPathExpression partsPath,
		List<String> parts
	) throws XmlException {
	
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		dbf.setIgnoringElementContentWhitespace(true);
	
		DocumentBuilder docBuilder;
		try {
		    docBuilder = dbf.newDocumentBuilder();
		} catch (ParserConfigurationException e) {
		    throw new XmlException("Kan ikke lage DocumentBuilder", e);
		}
		
		Document doc = docBuilder.newDocument();
		
		Element result = doc.createElement("frida");
		result.setAttribute("xmlns:xsi",
				    "http://www.w3.org/2001/XMLSchema-instance");
		result.setAttribute("xsi:noNamespaceSchemaLocation", skjemaURI);
		doc.appendChild(result);

		Element generert = doc.createElement("generert");
	
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
	
		generert.appendChild(doc.createTextNode(sdf.format(new Date())));
		result.appendChild(generert);
	
		for (String part : parts) {
		    Document merge = null;
		    try {
		    	merge = docBuilder.parse(new InputSource(new StringReader(part)));
		    } catch (Exception e) {
		    	throw new XmlException("Ugyldig XML funnet", e);
		    }
	
		    Node tree = null;
		    try {
		    	tree = (Node) partsPath.evaluate(merge, XPathConstants.NODE);
		    } catch (XPathExpressionException e) {
		    	throw new XmlException("Kan ikke finne angitte eleent", e);
		    }
		    tree = doc.importNode(tree, true);
		    result.appendChild(tree);
		}
	
		doc.setXmlStandalone(true);
		return docToString(doc);
	}
	
    /**
     * Lager String av XML Document.
     *
     * @param doc et XML Document
     * @return dokumentet som en String
     */
    private static String docToString(Document doc)
	throws XmlException {
		try {
		    Transformer tf = TransformerFactory.newInstance().newTransformer();
		    StringWriter sw = new StringWriter();
		    DOMSource source = new DOMSource(doc);
		    Result result = new StreamResult(sw);
		    tf.transform(source, result);
		    return sw.toString();
		} catch (Exception e) {
		    throw new XmlException("Kunne ikke konv. Document til streng", e);
		}
    }

}
